#include <avr/io.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "lcd_ks0066_4bit_io.h"
#include "led.h"
#include "taster.h"
#include "mbi5030.h"


//256 PWM-Stufen für jeden der 12 Kanäle
//16-Bit-PWM-Werte werden daraus aus dem 16-Bit-Array gezogen

uint8_t pwm_werte[KANAELE];

// wählbare Geschwindigkeiten für die Fading-Verläufe 
// Anzeige in der 2. Displayzeile

tempo_stufe fading_tempo_stufen[] PROGMEM = {
   { "Durchlaufzeit 3 min", 117 },
   { "Durchlaufzeit 2 min",  78 },
   { "Durchlaufzeit 90 sek", 59 },
   { "Durchlaufzeit 60 sek", 39 },
   { "Durchlaufzeit 40 sek", 26 },
   { "Durchlaufzeit 30 sek", 20 },
   { "Durchlaufzeit 20 sek", 13 },
   { "Durchlaufzeit 15 sek", 10 },
   { "Durchlaufzeit 10 sek",  7 },
   { "Durchlaufzeit 4 sek",   3 }
};

uint8_t tempo;
uint8_t farbe;

// wählbare farben für die Funktion konstante_farbe() für Wiedergabe konstanter Farben 
// Anzeige des Namens in der 2. Displayzeile

farb_set farb_sets[] PROGMEM = {
   { "Rot", 255, 0, 0, 255, 255, 255 },
   { "Gruen", 0, 255, 0, 255, 255, 255 },
   { "Blau", 0, 0, 255, 255, 255, 255 },
   { "Weiss", 255, 255, 255, ABGLEICH_ROT, ABGLEICH_BLAU, ABGLEICH_GRUEN },
   { "Gelb", 255, 255, 0, ABGLEICH_ROT, ABGLEICH_BLAU, ABGLEICH_GRUEN },
   { "Violett", 255, 0, 255, ABGLEICH_ROT, ABGLEICH_BLAU, ABGLEICH_GRUEN },
   { "Amber", 255, 255, 223, ABGLEICH_ROT, ABGLEICH_BLAU, ABGLEICH_GRUEN },
   { "Tuerkis", 0, 255, 255, ABGLEICH_ROT, ABGLEICH_BLAU, ABGLEICH_GRUEN },
   { "Orange", 255, 192, 0, ABGLEICH_ROT, ABGLEICH_BLAU, ABGLEICH_GRUEN }
};


//
// Funktion, welche die PWM-Stufe gleich läßt, eins höher nimmt oder eins reduziert
// -> Helligkeitswechsel in led_set 0->1, dann rauf, 1->0, dann runter
// -> wenn 1->1 oder 0->0, dann keine veränderung
//
void auf_oder_ab(uint8_t erstes, uint8_t zweites, uint8_t *pwm) {
    if (erstes != zweites) {
    	*pwm = (erstes > zweites)? *pwm-1 : *pwm+1;
    }
}

//
// Funktion, um die globalen Variablen im Modul led.c in main() zu initialisieren
//
void init_tempo_und_farbe(void) {
    tempo = 0;
    farbe = 0;
}

//
// Funktion, die zum Ausschalten aller LEDs dient
// wird in main() aufgerufen und macht alle LEDs dunkel, zeigt den Text im Display an
//
void alles_aus(void) {
    uint8_t i;
    
    lcd_clrscr();
    lcd_puts("ON/OFF druecken fuer");
    lcd_gotoxy(0,1);
    lcd_puts("Einschalten");
    
    for (i=0; i<KANAELE; i++) {
        pwm_werte[i] = 0x0000;
    }
    
    // PWM-Werte in die Register des MBI5030 schreiben
    mbi5030_write_row(pwm_werte,0);
}


//
// Funktion, welche die Fading-Verläufe aus den 12 Farbverläufen 
// (Folge von 0en und 1en, in den Strukturvariablen mit led_set deklariert)
// berechnet und nach gewähltem Fading Tempo zyklisch in die Register des MBI5030
// schrebt
// 
void fading(led_set *set)
{
    uint16_t zyklus;
    uint8_t i, j, anz_tempo_stufen;
    int8_t tast;
    tempo_stufe akt_tempo_stufe;
    
    //ermitteln, wieviele Elemente das Struct-Array fading_tempo_stufen hat
    
    anz_tempo_stufen = sizeof(fading_tempo_stufen)/sizeof(tempo_stufe);
    
    //aktuelle Tempostufe setzen und anzeigen
    
    memcpy_P(&akt_tempo_stufe,&fading_tempo_stufen[tempo],sizeof(tempo_stufe));
    lcd_gotoxy(0,1);
    lcd_leerzeile();
    lcd_gotoxy(0,1);
    lcd_puts(akt_tempo_stufe.name);

    //Startwerte für die PWM-Stufen je nach Kanalmaske setzen
    //Wenn die Maske mit '1' beginnt, dann Maximalwert (PWM-Stufen-1), sonst 0
    
    for (i=0; i<KANAELE; i++) {
       pwm_werte[i] = (set->kanaele[i][0] == 0)? 0 : PWM_STUFEN-1;
    }
    
    i=0;
    
    while (1) { //aeußere Schleife, so lang wie ein Kanalarray-1
	//innere Schleife, so lang wie die anzahl der dimmbaren Stufen-1 
	for (zyklus = 0; zyklus < PWM_STUFEN-1; zyklus++) {
	    // lokale Variable mit dem Inhalt der globalen "taster" (taster.c) belegen
	    tast = taster; 
	    // Auswerten, ob eine Taste gedrückt wurde
	    // KEY_SLOW, KEY_FAST werden verwertet und erhöhen/verringern das Fading-Tempo
	    // jeder andere Tastendruck führt zum Verlassen der Fading-Funktion -> Rückkehr zu main()
	    switch (tast)
            {
                case NO_TASTER:
                    break;
                case KEY_SLOW:
		    if (tempo == 0) {
			tempo = anz_tempo_stufen - 1;
		    }
		    else {
			tempo--;
		    }
		    memcpy_P(&akt_tempo_stufe,&fading_tempo_stufen[tempo],sizeof(tempo_stufe));
		    lcd_gotoxy(0,1);
		    lcd_leerzeile();
		    lcd_gotoxy(0,1);
		    lcd_puts(akt_tempo_stufe.name);
                    break;
	        case KEY_FAST:
		    if (tempo == anz_tempo_stufen - 1) {
			tempo = 0;
		    }
		    else {
			tempo++;
		    }
		    memcpy_P(&akt_tempo_stufe,&fading_tempo_stufen[tempo],sizeof(tempo_stufe));
		    lcd_gotoxy(0,1);
		    lcd_leerzeile();
		    lcd_gotoxy(0,1);
		    lcd_puts(akt_tempo_stufe.name);
                    break;
                case KEY_DOWN:
		    // Fade-Funktion verlassen, globale Variable "taster" bleibt, wird in main() ausgewertet
		    return;
                    break;
	        case KEY_UP:
		    // Fade-Funktion verlassen, globale Variable "taster" bleibt, wird in main() ausgewertet
		    return;
                    break;
	        case KEY_ON_OFF:
                    /* Taster "On/Off kurz gedrückt" */
		    // Fade-Funktion verlassen, globale Variable "taster" bleibt, wird in main() ausgewertet
		    return;
                    break;
	        case KEY_ON_OFF + TASTER_LONG:
                    /* Taster "On/Off lang gedrückt" */
		    // Fade-Funktion verlassen, globale Variable "taster" bleibt, wird in main() ausgewertet
		    return;
                    break;
                default:
                    break;
            } //Ende switch
		
	    if (tast != NO_TASTER) { // wenn "tast" (aus "taster" am Anf. von for (;;)) einen Wert hatte
	        taster = NO_TASTER;  // dann die globale variable "taster" (taster.c) zurücksetzen
	    }			     // damit in get_taster() eine neue Tastenauswertung freigegeben wird
	    
	    // PWM-Stufen in-/dekremenieren oder belassen, je nach Fading-Maske des jeweiligen Kanals
	    // das macht die Funktion auf_oder_ab()

	    for (j=0; j<KANAELE; j++) {
	        if (i < ANZ_VERLAUF-1) auf_oder_ab(set->kanaele[j][i], set->kanaele[j][i+1], &pwm_werte[j]);
	        if (i == ANZ_VERLAUF-1) auf_oder_ab(set->kanaele[j][i], set->kanaele[j][0], &pwm_werte[j]);
	    }
	    
	    // PWM-Werte in die Register des MBI5030 schreiben
	    mbi5030_write_row(pwm_werte, set->lin_log);
	    	    
	    _delay_ms(akt_tempo_stufe.anz_ms);
	}//Ende innere for-Schleife
	
	i++;
	if (i == ANZ_VERLAUF) i=0; //wenn Ende Kanalarray, dann weiter von vorne
    }//Ende aeußere while-Schleife
}//Ende fading()


// Errechnet die Dimmstufe 0 ... 255 für einen Farbwert mit zusätzlichem Farbabgleich
// Werte für abgleich_wert > 0 machen den PWM-Wert niedriger
// Das dient dazu Helligkeitsunterschiede der verschiedenen LEDs eines RGB-sets auszugleichen
// sonst wären keine definierten Mischfarben wie gelb oder türkis usw. möglich

uint8_t farbwert_mit_abgleich(uint8_t farbwert, uint8_t abgleich_wert) {

    uint8_t tmp;
    
    tmp = 255-abgleich_wert;
    return((farbwert > tmp) ? farbwert-tmp : 0);
    
} //Ende farbwert_mit_abgleich

// Funktion zum setzen eines konstanten Farbwertes = Mischung aus r, g, oder b für
// alle vier 3er-Kanalgruppen
// definiert in der struktur_variable vom typ farb_set

void farbset_aktivieren(farb_set *aktuelles_farb_set) {
	
    uint8_t i;
    farb_set akt_farb_set;
    
    memcpy_P(&akt_farb_set,aktuelles_farb_set,sizeof(farb_set));
    lcd_gotoxy(0,1);
    lcd_leerzeile();
    lcd_gotoxy(0,1);
    lcd_puts(akt_farb_set.name);
    // PWM-Werte für die Farben berechnen und in die Wertetabelle pwm_werte schreiben
    for (i=0; i<KANAELE; i++) {
        if (i%3==0) pwm_werte[i] = farbwert_mit_abgleich(akt_farb_set.rot,akt_farb_set.abgleich_rot);
        if (i%3==1) pwm_werte[i] = farbwert_mit_abgleich(akt_farb_set.blau,akt_farb_set.abgleich_blau);
        if (i%3==2) pwm_werte[i] = farbwert_mit_abgleich(akt_farb_set.gruen,akt_farb_set.abgleich_gruen);
    }    
	
    // PWM-Werte in die Register des MBI5030 schreiben
    mbi5030_write_row(pwm_werte,1);
}

// Funtion, welche die in farb_set farb_sets{} definierten Farben widergibt
// Auswahl mit den Tasten KEY_SLOW und KEY_FAST

void konstante_farbe(void) {
    uint8_t anz_farb_sets;
    int8_t tast;
    
    anz_farb_sets = sizeof(farb_sets)/sizeof(farb_set);
    
    farbset_aktivieren(&farb_sets[farbe]);

    for (;;) { //Endlosschleife, wird nur verlassen, wenn KEY_UP oder KEY_DOWN gedrückt wird
	// lokale Variable mit dem Inhalt der globalen "taster" (taster.c) belegen
	tast = taster; 

	// Auswerten, ob eine Taste gedrückt wurde
	// KEY_SLOW, KEY_FAST werden verwertet und wählen die vorherige/nächste Farbe aus
	// jeder andere Tastendruck führt zum Verlassen der Funktion -> Rückkehr zu main()
	switch (tast)
        {
            case NO_TASTER:
                break;
            case KEY_SLOW:
	        if (farbe == 0) {
		    farbe = anz_farb_sets - 1;
		}
		else {
		    farbe--;
		}
		farbset_aktivieren(&farb_sets[farbe]);
                break;
	    case KEY_FAST:
		if (farbe == anz_farb_sets - 1) {
		    farbe = 0;
		}
		else {
		    farbe++;
		}
		farbset_aktivieren(&farb_sets[farbe]);
                break;
            case KEY_DOWN:
		// Fade-Funktion verlassen, globale "taster" bleibt, wird in main() ausgewertet
		return;
                break;
	    case KEY_UP:
		// Fade-Funktion verlassen, globale "taster" bleibt, wird in main() ausgewertet
		return;
                break;
	    case KEY_ON_OFF:
                /* Taster "On/Off kurz gedrückt" */
		// Fade-Funktion verlassen, globale "taster" bleibt, wird in main() ausgewertet
		return;
                break;
	    case KEY_ON_OFF + TASTER_LONG:
                /* Taster "On/Off lang gedrückt" */
		// Fade-Funktion verlassen, globale "taster" bleibt, wird in main() ausgewertet
		return;
                break;
            default:
                break;
        } //Ende switch
		
	if (tast != NO_TASTER) { // wenn "tast" (aus "taster" am Anf. von for (;;)) einen Wert hatte
	    taster = NO_TASTER;  // dann die globale variable "taster" (taster.c) zurücksetzen
	}			     // damit in get_taster() eine neue Tastenauswertung freigegeben wird
    } //Ende for (;;)
}//Ende konstante_farbe()
