#include <stdlib.h>
#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "lcd_ks0066_4bit_io.h"
#include "taster.h"
#include "led.h"
#include "mbi5030.h"

#define EIN 1
#define AUS 0

//-------------------------------------------------------------------------------------------------------
// Aufbau der Datenstrukturen
//-------------------------------------------------------------------------------------------------------

// Datensätze für Fading-effekte

led_set fading_set_RGB_com PROGMEM = {
    "RGB",
     { {1,1,0,0,0,1}, //rot   \
       {0,0,0,1,1,1}, //blau   > Kanäle 1-3, also erste RGB-Gruppe
       {0,1,1,1,0,0}, //grün  /
       {1,1,0,0,0,1}, 
       {0,0,0,1,1,1}, //          > dto. 4-6, also zweite RGB-Gruppe
       {0,1,1,1,0,0},
       {1,1,0,0,0,1},
       {0,0,0,1,1,1}, //	  > und die dritte
       {0,1,1,1,0,0},
       {1,1,0,0,0,1},
       {0,0,0,1,1,1}, //          > und vierte
       {0,1,1,1,0,0}  },
       0
};

led_set fading_set_subtr_com PROGMEM = {
     "RGB subtraktiv",
     { {1,0,1,1,1,1},
       {1,1,1,1,1,0},
       {1,1,1,0,1,1},
       {1,0,1,1,1,1},
       {1,1,1,1,1,0},
       {1,1,1,0,1,1},
       {1,0,1,1,1,1},
       {1,1,1,1,1,0},
       {1,1,1,0,1,1},
       {1,0,1,1,1,1},
       {1,1,1,1,1,0},
       {1,1,1,0,1,1}  },
       0
};

led_set fading_set_RGB_seq PROGMEM = {
    "RGB sequentiell",
     { {1,1,0,0,0,1},
       {0,0,0,1,1,1},
       {0,1,1,1,0,0},
       {1,0,0,0,1,1},
       {0,0,1,1,1,0},
       {1,1,1,0,0,0},
       {0,0,0,1,1,1},
       {0,1,1,1,0,0},
       {1,1,0,0,0,1},
       {0,0,1,1,1,0},
       {1,1,1,0,0,0},
       {1,0,0,0,1,1}  },
       0
};

led_set fading_set_subtr_seq PROGMEM = {
     "RGB subtraktiv seq.",
     { {1,0,1,1,1,1},
       {1,1,1,1,1,0},
       {1,1,1,0,1,1},
       {0,1,1,1,1,1},
       {1,1,1,1,0,1},
       {1,1,0,1,1,1},
       {1,1,1,1,1,1},
       {1,1,1,0,1,1},
       {1,0,1,1,1,1},
       {1,1,1,1,1,0},
       {1,1,0,1,1,1},
       {0,1,1,1,1,1}  },
       0
};

led_set fading_set_RGB_einz_com PROGMEM = {
    "RGB einz. schwellend",
     { {0,1,0,0,0,0},
       {0,0,0,0,0,1},
       {0,0,0,1,0,0},
       {0,1,0,0,0,0},
       {0,0,0,0,0,1},
       {0,0,0,1,0,0},
       {0,1,0,0,0,0},
       {0,0,0,0,0,1},
       {0,0,0,1,0,0},
       {0,1,0,0,0,0},
       {0,0,0,0,0,1},
       {0,0,0,1,0,0}  },
       1
};

led_set akt_led_set; // Variable in welcher das aktuelle Set aus dem Flash zwischengespeichert wird

/* Bei 8MHz Grundtakt und Prescaler 1:8 läuft Timer0 alle 256µs über.
 * Um auf rund 10ms zu kommen, rufen wir get_taster nur
 * jedes 39. mal auf. */

ISR (TIMER0_OVF_vect)
{
    static uint8_t count_ovl0;
    uint8_t ovl0 = count_ovl0+1;

    if (ovl0 >= 39)
    {
        get_taster (KEY_SLOW, KEY_SLOW_PIN & (1<<KEY_SLOW_PINNR));
        get_taster (KEY_FAST, KEY_FAST_PIN & (1<<KEY_FAST_PINNR));
        get_taster (KEY_DOWN, KEY_DOWN_PIN & (1<<KEY_DOWN_PINNR));
	get_taster (KEY_UP, KEY_UP_PIN & (1<<KEY_UP_PINNR));
	get_taster (KEY_ON_OFF, KEY_ON_OFF_PIN & (1<<KEY_ON_OFF_PINNR));

        ovl0 = 0;
    }

    count_ovl0 = ovl0;
}


int main(void)
{

    int8_t tast;
    uint8_t led_programm_pos = 0, on_off = AUS;

    /* initialize display, cursor off */
    lcd_init(LCD_DISP_ON);
    
    /* initialisiere die Taster */
    taster_init();
    
    /* initialisiere den MBI5030 */
    mbi5030_init();
    
    /* initialisiere die Durchlaufzeit für Fading, globale Variable "tempo" in led.c */
    init_tempo_und_farbe();

    // Interrupts aktivieren
    sei();
    
    alles_aus();
    
    for (;;) { // loop forever
	tast = taster; //taster kann auch einen Tastendruck enthalten, welcher in den Funktionen
	               //fading() oder konstante_farbe() grdückt wurde
		
	switch (tast)
        {
            case NO_TASTER:
                break;
            case KEY_DOWN:
                // Taster "Programmwahl abwärts" 
                if (led_programm_pos == 0) { // wenn schon an Pos. "0", dann auf oberes Ende setzen
                    led_programm_pos = ANZ_PROGRAMME - 1;
                }
                else { // also > 0 dann dekrementieren
                    led_programm_pos--;
                }
                break;
	    case KEY_UP:
		// Taster "Programmwahl aufwärts" 
		if (led_programm_pos == ANZ_PROGRAMME - 1) { // wenn schon ganz oben, dann auf "0" springen
		    led_programm_pos = 0;
		}
		else {
		    led_programm_pos++;
		}
                break;
	    case KEY_ON_OFF + TASTER_LONG:
                // Taster "On/Off lang gedrückt"
                if (on_off == AUS) {
                    on_off = EIN;
                    lcd_clrscr();
                }
                else {
                    on_off = AUS;
                    alles_aus();
                }
                break;
            default:
                //Taster KEY_FAST oder KEY_SLOW werden hier nicht gefragt
                //-> Verwendung in den Fade-/Farbfunktionen (led.c)
                break;
        } //Ende switch
		
	if (tast != NO_TASTER) { // wenn "tast" (zugewiesen aus "taster" am Anf. von for (;;)) einen Wert hatte
	    taster = NO_TASTER;  // dann die globale variable "taster" (taster.c) zurücksetzen
	}			 // damit in get_taster() eine neue Tastenauswertung freigegeben wird
	
	if (on_off == EIN) {
   	    // nun wird anhand der Programmposition in die ensprechende Fade- oder Farbwechselfunktion
	    // gesprungen.
	    // Dort werden die Tasten KEY_SLOW und KEY_FAST verwendet.
	    // Die Tasten KEY_DOWN, KEY_UP oder KEY_ON_OFF führen zum Rücksprung aus der jeweiligen Funktion
	    switch (led_programm_pos)
            {
                case 0:
		    lcd_clrscr();
		    memcpy_P(&akt_led_set,&fading_set_RGB_com,sizeof(led_set));
		    lcd_puts(akt_led_set.name);
		    fading(&akt_led_set);
                    break;
                case 1:
		    lcd_clrscr();
		    memcpy_P(&akt_led_set,&fading_set_subtr_com,sizeof(led_set));
		    lcd_puts(akt_led_set.name);
		    fading(&akt_led_set);
                    break;
                case 2:
		    lcd_clrscr();
		    memcpy_P(&akt_led_set,&fading_set_RGB_seq,sizeof(led_set));
		    lcd_puts(akt_led_set.name);
		    fading(&akt_led_set);
                    break;
                case 3:
		    lcd_clrscr();
		    memcpy_P(&akt_led_set,&fading_set_subtr_seq,sizeof(led_set));
		    lcd_puts(akt_led_set.name);
		    fading(&akt_led_set);
                    break;
                case 4:
		    lcd_clrscr();
		    memcpy_P(&akt_led_set,&fading_set_RGB_einz_com,sizeof(led_set));
		    lcd_puts(akt_led_set.name);
		    fading(&akt_led_set);
                    break;
                case 5:
		    lcd_clrscr();
		    lcd_puts("konstante Farbe");
                    konstante_farbe();
                    break;
                default:
                    break;
	    } // Ende switch (led_programm_pos)
	}//Ende if (on_off == EIN)
    } // Ende for(;;)

} // Ende   int main(void)

