#include <inttypes.h>
#include <avr/io.h>
#include "taster.h"

volatile int8_t taster = NO_TASTER;

taste_t tasten[NUM_TASTER];

/*
 * initialisiert die Portd/DDRs/Pins für die Taster
 */
void taster_init (void)
{
    KEY_SLOW_DDR &= ~(1<<KEY_SLOW_DDRNR); //Tasterport als Eingang
    KEY_SLOW_PORT |= (1<<KEY_SLOW_PINNR); //Pullup aktivieren
    
    KEY_FAST_DDR &= ~(1<<KEY_FAST_DDRNR);
    KEY_FAST_PORT |= (1<<KEY_FAST_PINNR);
    
    KEY_DOWN_DDR &= ~(1<<KEY_DOWN_DDRNR);
    KEY_DOWN_PORT |= (1<<KEY_DOWN_PINNR);
    
    KEY_UP_DDR &= ~(1<<KEY_UP_DDRNR);
    KEY_UP_PORT |= (1<<KEY_UP_PINNR);
    
    KEY_ON_OFF_DDR &= ~(1<<KEY_ON_OFF_DDRNR);
    KEY_ON_OFF_PORT |= (1<<KEY_ON_OFF_PINNR);
    
    // Tasten konfigurieren
	
    tasten[KEY_SLOW].mode = TM_REPEAT;
    tasten[KEY_FAST].mode = TM_REPEAT;
    tasten[KEY_DOWN].mode = TM_SHORT;
    tasten[KEY_UP].mode = TM_SHORT;
    tasten[KEY_ON_OFF].mode = TM_LONG;

    /* Timer0 mit Prescaler 1:8 starten */
    TCCR0 = 1 << CS01;
 
    /* im Register TimerInterruptMask (=TIMSK) Timer0-Overflow-Interrupt aktivieren */
    TIMSK |= (1 << TOIE0);
}

/*
 * Aktualisiert 'taster', falls taster==NO_TASTER
 * num:  Nummer des Tasters von 0...NUM_TASTER-1
 * TASTER_LEVEL=1:
 *    tast: ==0 falls der Taster gerade nicht gedrueckt wird
 *    tast: !=0 falls der Taster gerade gedrueckt wird
 * TASTER_LEVEL=0:
 *    tast: !=0 falls der Taster gerade nicht gedrueckt wird
 *    tast: ==0 falls der Taster gerade gedrueckt wird
 */
void get_taster (const uint8_t num, uint8_t tast) 
{
    const taste_t * const ptast = & tasten[num];
    const uint8_t taster_old = ptast->old;
    uint8_t pressed, press, release, mode, delay;

#if TASTER_LEVEL
    tast = !!tast;
#else
    tast = !tast;
#endif
 
    /* Was wurde gedrueckt/losgelassen...? */
 
    /* Taster bleibt gedrueckt */
    pressed =  taster_old &  tast;
    /* Taster neu gedrueckt */
    press   = ~taster_old &  tast;
    /* Taster losgelassen */
    release =  taster_old & ~tast;

    /* ptast->old = tast;
     * Der Cast dient zum Wegwerfen des 'const' Qualifiers. 
     * Die Komponente wurde als 'const' qualifiziert, 
     * damit es einen Fehler gibt, wenn versucht wird,
     * ihren Wert von aussen zu aendern (private). */
    *((uint8_t *) & ptast->old) = tast;
 
    tast = NO_TASTER;
 
    mode  = ptast->mode;
    delay = ptast->delay;
 
    if (press)
    {
        if (mode != TM_LONG)
            tast = num;
   
        delay = 0;
    }
    else if (pressed)
    {
        if (delay < 0xfe)
            delay++;
    }
    else if (release)
    {
        if (mode == TM_LONG && delay != 0xff)
            tast = num;
    }
 
    if (mode == TM_LONG)
    {
        if (delay == TASTER_DELAY_LONG)
        {
            tast = TASTER_LONG + num;
            delay = 0xff;
        }
    }
    else if (mode == TM_REPEAT)
    {
        if (delay == TASTER_REPEAT_DELAY)
        {
            tast = num;
            delay = TASTER_REPEAT_DELAY - TASTER_REPEAT;
        }
    }

    if (taster == NO_TASTER)
        taster = tast;
  
    /* siehe oben */
    *((uint8_t *) & ptast->delay) = delay;
}
