#ifndef _TASTER_H_

#define _TASTER_H_
#include <stdint.h>

// Tastennamen per Definition Pin-Nummern zuordnen

#define KEY_SLOW  0
#define KEY_FAST 1
#define KEY_DOWN  2
#define KEY_UP    3
#define KEY_ON_OFF 4

// Eingangspins für Taster Definieren

#define KEY_SLOW_DDR DDRC
#define KEY_SLOW_DDRNR DDC0
#define KEY_SLOW_PORT PORTC
#define KEY_SLOW_PIN PINC
#define KEY_SLOW_PINNR PC0

#define KEY_FAST_DDR DDRC
#define KEY_FAST_DDRNR DDC1
#define KEY_FAST_PORT PORTC
#define KEY_FAST_PIN PINC
#define KEY_FAST_PINNR PC1

#define KEY_DOWN_DDR DDRB
#define KEY_DOWN_DDRNR DDB0
#define KEY_DOWN_PORT PORTB
#define KEY_DOWN_PIN PINB
#define KEY_DOWN_PINNR PB0

#define KEY_UP_DDR DDRB
#define KEY_UP_DDRNR DDB1
#define KEY_UP_PORT PORTB
#define KEY_UP_PIN PINB
#define KEY_UP_PINNR PB1

#define KEY_ON_OFF_DDR DDRB
#define KEY_ON_OFF_DDRNR DDB2
#define KEY_ON_OFF_PORT PORTB
#define KEY_ON_OFF_PIN PINB
#define KEY_ON_OFF_PINNR PB2

/* Wert fuer taster, wenn nichts gedrückt wurde */
#define NO_TASTER (-1)

/* Maximale Anzahl der Taster */
#define NUM_TASTER 5

/* 0 --> Taster sind low-aktiv */
/* 1 --> Taster sind high-aktiv */
#define TASTER_LEVEL 0

/* Dieser Offset wird zur Tasten-Nummer addiert, */
/* wenn eine Taste lange gedrückt wurde */
#define TASTER_LONG 16

/* Zeitverzügerung (in Ticks), bis zum Beginn von auto-repeat */
#define TASTER_REPEAT_DELAY (60)

/* Zeitverzügerung (in Ticks), bis zum nächsten auto-repeat */
#define TASTER_REPEAT       (15)

/* Ab dieser Dauer wird der Tastendruck 'lange' */
#define TASTER_DELAY_LONG   (80)

typedef struct 
{
    /* private */
    const uint8_t delay, old;

    /* Mode des Tasters aus: TM_SHORT, TM_LONG, TM_REPEAT */
    uint8_t mode;
} taste_t;

extern taste_t tasten[];

/* In dieser Variable kann abgefragt werden, welche Taste gedrückt wurde.
   --> NO_TASTER:
       es wurde nichts gedrückt
   --> 0..NUM_TASTER-1:
       Taster Numero 'taster' wurde (kurz) gedrückt
   --> TASTER_LONG ... TASTER_LONG + NUM_TASTER-1:
       Taster Numero 'taster-TASTER_LONG' wurde lange gedrückt
*/   
extern volatile int8_t taster;

extern void get_taster (const uint8_t num, uint8_t tast);

extern void taster_init (void);

enum
{
    TM_SHORT,
    TM_LONG,
    TM_REPEAT
};

#endif /* _TASTER_H_ */
